from infinishell.infinibox.commands.cluster import SORTABLE_FIELDS

from munch import Munch

from infinishell.infishell import commandgroup, command, render
from infinishell.infishell.types import PreferredControllers, YesNo, Str
from infinishell.infishell.utils import strlist, NOTHING
from infinishell.infishell.exceptions import CommandAborted, BadArguments,CommandFailed,BadValue

from ....utils import input_password
from ..types import (
    SmbGroup,
    SmbUser,
    Hostname,
    SmbPrivileges,
    SmbGroupName,
    SmbActiveDirectoryDomain,
    SmbUserName,
    )
from ..options import QueryOptions
from .. import EntityCommands, roles, render_collection
from . import ConfigCommandsBase


@commandgroup(parent=ConfigCommandsBase, prefix="nas")
class NasCommands(ConfigCommandsBase):
    pass

@commandgroup(parent=NasCommands, prefix="active_directory")
class ADCommands(ConfigCommandsBase, EntityCommands):
    ENTITY_TYPE = SmbActiveDirectoryDomain

    @command(
        name="query",
        description="Show Active Directory domain details",
        )
    @render_collection(
        individually=True,
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def details(self, _api):
        domain_result = SmbActiveDirectoryDomain.get_all(_api)
        domain_query = {
            "domain": domain_result.domain,
            "preferred_controllers": domain_result.preferred_ips
        }

        return [domain_query]

    @command(
        description="Join an Active Directory domain",
        types=(Str, Str, PreferredControllers.List, Str),
        )
    @roles.INFINIDAT.ADMIN
    def join(self, _api, domain, username, preferred_controllers, password=None):
        if password is None:
            password = input_password('Please enter a password for user {}: '.format(username),
            interaction_exc_type=BadArguments, interaction_error="Please specify a password")
             # url# new type
        data = dict(
            domain=domain,
            org_unit="",
            username=username,
            password=password,
            preferred_ips=preferred_controllers,)
        result = _api.post("/activedirectory/domains", data)

    @command(
        description="Change Active Directory settings",
        types=(PreferredControllers.List,),
        )
    @roles.INFINIDAT.ADMIN
    def set_preferred_controllers(self, _api, preferred_controllers):
        result = _api.post("/activedirectory/domains/set_preferred_ips", preferred_controllers)

    @command(
        description="Leave the Active Directory domain",
        types=(Str, Str)
        )
    @roles.INFINIDAT.ADMIN
    def leave(self, _api, username, password=None):
        if password is None:
            password = input_password('Please enter a password for user {}: '.format(username),
            interaction_exc_type=BadArguments, interaction_error="Please specify a password")
        data = dict(
            username=username,
            password=password,
        )
        result = _api.post("/activedirectory/domains/leave", data)


@commandgroup(parent=NasCommands, prefix="smb_users")
class SmbUserCommands(ConfigCommandsBase, EntityCommands):
    ENTITY_TYPE = SmbUser

    @command(
        description="Create a new SMB local user",
        types=(SmbUserName, Str, YesNo, SmbGroup, SmbGroup.List, SmbPrivileges.List),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def create(self, _api, name, password=None, enabled=True, primary_group=None, groups=None, privileges=["SECHANGENOTIFYPRIVILEGE"]):
        if password is None:
            password = input_password('Please enter a password for user {}: '.format(name),
            interaction_exc_type=BadArguments, interaction_error="Please specify a password")
             # url# new type
        data = Munch(
            name = name,
            password = password,
            enabled = enabled,
            primary_group_id = primary_group,
            group_ids = groups,
            privileges = privileges,
        )
        return self._create(_api, **data)

    @command(
        description="Change the password for an SMB local user",
        types=(SmbUser, Str),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def change_password(self, _api, user, password=None):
        if password is None:
            password = input_password('New password: ',
            interaction_exc_type=BadArguments, interaction_error="Please specify a password")
        if password != input_password(prompt="Re-enter new password: "):
            raise CommandFailed("Passwords differ from one another, please try again")
        if not password.strip():
            raise CommandAborted("Password is empty, aborting")
        self._update(_api, user, password=password)
        return self._result_message(user, "password changed")

    @command(
        description="Change the primary group for an SMB local user",
        types=(SmbUser, SmbGroup),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def set_primary_group(self, _api, user, primary_group):
        self._update(_api, user, primary_group_id=primary_group)
        return self._result_message(user, "updated")

    @command(
        description="Add the SMB local user as member in the group",
        types=(SmbUser, SmbGroup.List),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def add_group(self, _api, user, group):
        all_groups = user.groups + group
        self._update(_api, user, group_ids=all_groups)
        return self._result_message(user, "updated")

    @command(
        description="Remove the SMB local user from the group",
        types=(SmbUser, SmbGroup),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def remove_group(self, _api, user, group):
        if group not in user.groups:
            raise BadValue('SMB user "{}" has no such group: {}'.format(user.name,group.name))
        user.groups.remove(group)
        self._update(_api, user, group_ids=user.groups)
        return self._result_message(user, "updated")
    
    @command(
        description="Change privileges for an SMB local group",
        types=(SmbUser, SmbPrivileges.List),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def set_privileges(self, _api, user, privileges):
        self._update(_api, user, privileges=privileges)
        return self._result_message(user, "updated")

    QUERY_FIELDS = strlist("name sid enabled primary_group groups privileges")
    SORTABLE_FIELDS = strlist("name enabled primary_group")
    QUERY_SUBSTITUTES = dict(
        name="user"
        )

    @command(
        description="List existing SMB local users",
        types=(SmbUser, YesNo, SmbGroup),
        options=QueryOptions(fields=QUERY_FIELDS,substitute=QUERY_SUBSTITUTES ,detailed=True, sortable=SORTABLE_FIELDS),
        )
    @render(
        fields=QUERY_FIELDS,
        substitute=QUERY_SUBSTITUTES,
        detailed_individually=True,
        )

    @roles.INFINIDAT.ADMIN.POOLADMIN
    def query(self, _api, user=None, enabled=None, primary_group=None):
        return self._query(_api, user, _hook=self._query_hook, enabled=enabled, primary_group_id=primary_group)

    def _query_hook(self, users):
        for user in users:
            user.primary_group= user.primary_group.name if hasattr(user, "primary_group") else "N/A"
            user.groups = list(map(lambda group: group.name, user.groups))
            yield user

    @command(
        description="Disable an SMB local user",
        types=(SmbUser,),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def disable(self, _api, user):
        if not user.enabled:
            return self._result_message(user, "is already disabled")
        self._update(_api, user, enabled=False)
        return self._result_message(user, "disabled")

    @command(
        description="Disable an SMB local user",
        types=(SmbUser,),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def enable(self, _api, user):
        if user.enabled:
            return self._result_message(user, "is already enabled")
        self._update(_api, user, enabled=True)
        return self._result_message(user, "enabled")

    @command(
        description="Delete an SMB local user",
        types=(SmbUser,),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def delete(self, _api, user):
        return self._delete(_api, user)



@commandgroup(parent=NasCommands, prefix="smb_groups")
class SmbGroupCommands(ConfigCommandsBase, EntityCommands):
    ENTITY_TYPE = SmbGroup

    @command(
        description="Create a new SMB local group",
        types=(SmbGroupName, SmbPrivileges.List, Str.List),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def create(self, _api, name, privilege=["SECHANGENOTIFYPRIVILEGE"], domain_members=None):
        data = Munch(
            name = name,
            privileges = privilege,
            domain_members = domain_members,
        )
        return self._create(_api, **data)


    @command(
        description="Change privileges for an SMB local group",
        types=(SmbGroup, SmbPrivileges.List),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def set_privileges(self, _api, group, privileges):
        self._update(_api, group, privileges=privileges)


    @command(
        description="Change the AD domain members for an SMB local group",
        types=(SmbGroup, Str.List),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def set_domain_members(self, _api, group, domain_members):
        self._update(_api, group, domain_members=domain_members)


    @command(
        description="Delete an SMB local group",
        types=(SmbGroup,),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def delete(self, _api, group):
        return self._delete(_api, group)


    QUERY_FIELDS = strlist("name privileges domain_members")
    SORTABLE_FIELDS = strlist("name")
    QUERY_SUBSTITUTES = dict(
        name="group"
        )

    @command(
        description="List existing SMB local groups",
        types=(SmbGroup,),
        options=QueryOptions(fields=QUERY_FIELDS, substitute=QUERY_SUBSTITUTES, detailed=True, sortable=SORTABLE_FIELDS),
        )
    @render(
        fields=QUERY_FIELDS,
        substitute=QUERY_SUBSTITUTES,
        detailed_individually=True,
    )

    @roles.INFINIDAT.ADMIN.POOLADMIN
    def query(self, _api, group=None):
        return self._query(_api, group)
