from functools import partial
from infinishell.infishell import commandgroup, command
from infinishell.infishell.utils import strlist
from infinishell.infishell.types import Str
from . import render_collection, EntityCommands, roles, LazyIdNameMapping, LazyEntityMapping
from .types import (
    ReplicationGroup,
    Vvol,
    VvolPool,
    Vm,
    Name,
    ReplicatedLocalEntity,
    RgReplicas,
    ReplicationLinkBySystemName,
    Pool,
    TimeDeltaOrNone,
    )
from .options import QueryOptions
from .utils import get_target_pool, format_savings
from ..features import feature_command
from ..remote import get_remote_system

class RemoteEntity(Str):
    def get_description(self):
        return "Name of a remote entity (auto-completion unavailable)"


QUERY_FIELDS = strlist("name pool_name members_count replication_role")
RG_REPLICAS_QUERY_FIELDS = strlist("replication_group_id role replication_type sync_interval rpo_value pool_name")
DETAILED_QUERY_FIELDS = QUERY_FIELDS
SORTABLE_FIELDS = strlist("name")

MEMBER_QUERY_FIELDS = strlist("name rg vvol_type vm size used allocated")
MEMBER_QUERY_DETAILED_QUERY_FIELDS = MEMBER_QUERY_FIELDS +  strlist("type thin data_reduction allocated tree_allocated pool_name")
SUBSTITUTE = dict(pool_name="pool", tree_allocated="snapshots", members_count="members")

@commandgroup(prefix="rg")
class RgCommands(EntityCommands):
    ENTITY_TYPE = ReplicationGroup

    @feature_command(
        feature='vvol_replication',
        status=True,
        description="Create a new vVol replication group.",
        types=(Name, VvolPool),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def create(self, _api, name,  pool=None):
        pool = get_target_pool(_api, pool, "VVOL")
        return self._create(_api,
            name=name,
            pool_id=pool
            )

    @feature_command(
        feature='vvol_replication',
        status=True,
        description="List existing vVol replication groups",
        types=(ReplicationGroup.List, VvolPool),
        options=QueryOptions(fields=QUERY_FIELDS, sortable=SORTABLE_FIELDS, substitute=SUBSTITUTE, detailed=True),
        substitute=SUBSTITUTE,
    )
    @render_collection(
        entity=ReplicationGroup,
        fields=QUERY_FIELDS,
        detailed_fields=DETAILED_QUERY_FIELDS,
        detailed_individually=True,
        substitute=SUBSTITUTE,
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN.READONLY
    def query(self, _api, rg=None, pool_name=None):
        pool_names_by_id = LazyIdNameMapping(VvolPool, _api)
        lazy_replications_by_id = LazyEntityMapping(RgReplicas, _api, fields=["role"], mapping_key="replication_group_id")
        pool_names_by_id = LazyIdNameMapping(VvolPool, _api)
        return self._query(_api, rg, pool_id=pool_name, _hook=partial(self._query_hook, pool_names_by_id=pool_names_by_id, lazy_replications_by_id=lazy_replications_by_id))

    def _query_hook(self, rgs, pool_names_by_id, lazy_replications_by_id=None):
        for rg in rgs:
            rg.replication_role = None
            if lazy_replications_by_id and rg.is_replicated:
                rg.replication_role = lazy_replications_by_id[rg.id].role
            rg.pool_name = pool_names_by_id[rg.pool_id]
            yield rg

    @feature_command(
        feature='vvol_replication',
        status=True,
        description="Rename a vVol replication group",
        types=(ReplicationGroup, Name),
        hidden_substitue=dict(name="rg"),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def rename(self, _api, rg, new_name):
        return self._rename(_api, rg, new_name)

    @feature_command(
        feature='vvol_replication',
        status=True,
        description="Delete a vVol replication group",
        types=(ReplicationGroup,),
        hidden_substitue=dict(name="rg"),
    )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def delete(self, _api, rg):
        _api.delete(u"replication_groups/{}".format(rg.id))
        return self._result_message(rg, "deleted")

    @feature_command(
        feature='vvol_replication',
        status=True,
        description="Remove a vVol from a vVol replication group",
        types=(ReplicationGroup, Vvol),
        hidden_substitue=dict(name="rg"),
        )
    @roles.INFINIDAT.ADMIN
    def remove_member(self, _api, rg, member):
         _api.delete(u"replication_groups/{}/members/{}".format(rg.id, member.id))
         return ("vVol '{}' removed from replication group '{}'".format(member.name, rg.name))

    @feature_command(
        feature='vvol_replication',
        status=True,
        description="List existing vVols in a vVol replication groups.",
        types=(ReplicationGroup.List,),
        options=QueryOptions(fields=MEMBER_QUERY_FIELDS, sortable=SORTABLE_FIELDS, substitute=SUBSTITUTE, detailed=True),
        substitute=SUBSTITUTE,
    )
    @render_collection(
        entity=ReplicationGroup,
        fields=MEMBER_QUERY_FIELDS,
        detailed_fields=MEMBER_QUERY_DETAILED_QUERY_FIELDS,
        detailed_individually=True,
        substitute=SUBSTITUTE,
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN.READONLY
    def member_query(self, _api, rg=None):
        vm_names_by_id = LazyIdNameMapping(Vm, _api)
        rg_names_by_id = LazyIdNameMapping(ReplicationGroup, _api)
        if rg is None:
            filter = _api.create_filter().is_not_null('replication_group_id')
            return self._query(_api,  replication_group_id=rg,  _entity_type=Vvol, _filter=filter, _hook=partial(self._members_query_hook, rg_names_by_id=rg_names_by_id, vm_names_by_id=vm_names_by_id))
        return self._query(_api,  replication_group_id=rg,  _entity_type=Vvol, _hook=partial(self._members_query_hook, rg_names_by_id=rg_names_by_id, vm_names_by_id=vm_names_by_id))

    def _members_query_hook(self, vvols, vm_names_by_id, rg_names_by_id):
        for vvol in vvols:
            vvol.rg = rg_names_by_id[vvol.replication_group_id] if vvol.replication_group_id else 'N/A'
            vvol.vm = vm_names_by_id[vvol.vm_id]
            vvol.thin = vvol.prov_type == "THIN"
            vvol.data_reduction = format_savings(vvol)
            yield vvol


